/* -------------------------------------------------------

This shader implements support for 4 point lights placed as
OpenGL GL_LIGHT1 through GL_LIGHT4.

--------------------------------------------------------- */

uniform sampler2D tex;

varying vec3 vNormal;
varying vec4 vPosition;

void main()
{
	//Compute the globalAmbient term
	vec4 color = gl_LightModel.ambient * gl_FrontMaterial.ambient * texture2D(tex, gl_TexCoord[0].st);
	vec4 diffuse;
	vec4 ambient;
	vec3 lightDir[4];
	float dist[4];
	vec3 n;
	float NdotL;
	float att = 0.0;
	
	//Compute light directions
	lightDir[0] = vec3(vPosition - gl_LightSource[1].position);
	lightDir[1] = vec3(vPosition - gl_LightSource[2].position);
	lightDir[2] = vec3(vPosition - gl_LightSource[3].position);
	lightDir[3] = vec3(vPosition - gl_LightSource[4].position);
	
	//Compute the distance to the light source to a varying variables
	dist[0] = length(lightDir[0]);
	dist[1] = length(lightDir[1]);
	dist[2] = length(lightDir[2]);
	dist[3] = length(lightDir[3]);
	
	//A fragment shader can't write a varying variable, hence we need
	//a new variable to store the normalized interpolated normal
	n = normalize(vNormal);
	
	//Add contribution from each light	
	diffuse       = gl_FrontMaterial.diffuse * gl_LightSource[1].diffuse;
	ambient       = gl_FrontMaterial.ambient * gl_LightSource[1].ambient;
	NdotL         = -min(dot(n,normalize(lightDir[0])),0.0);
	att           = 1.0 - gl_LightSource[1].linearAttenuation * dist[0];
	att           = sqrt(att);
	att           = max(0.0, att);
	color        += att * att * (diffuse * NdotL + ambient);
	
	diffuse       = gl_FrontMaterial.diffuse * gl_LightSource[2].diffuse;
	ambient       = gl_FrontMaterial.ambient * gl_LightSource[2].ambient;
	NdotL         = -min(dot(n,normalize(lightDir[1])),0.0);
	att           = 1.0 - gl_LightSource[2].linearAttenuation * dist[1];
	att           = sqrt(att);
	att           = max(0.0, att);
	color        += att * att * (diffuse * NdotL + ambient);
	
	diffuse       = gl_FrontMaterial.diffuse * gl_LightSource[3].diffuse;
	ambient       = gl_FrontMaterial.ambient * gl_LightSource[3].ambient;
	NdotL         = -min(dot(n,normalize(lightDir[2])),0.0);
	att           = 1.0 - gl_LightSource[3].linearAttenuation * dist[2];
	att           = sqrt(att);
	att           = max(0.0, att);
	color        += att * att * (diffuse * NdotL + ambient);
	
	diffuse       = gl_FrontMaterial.diffuse * gl_LightSource[4].diffuse;
	ambient       = gl_FrontMaterial.ambient * gl_LightSource[4].ambient;
	NdotL         = -min(dot(n,normalize(lightDir[3])),0.0);
	att           = 1.0 - gl_LightSource[4].linearAttenuation * dist[3];
	att           = sqrt(att);
	att           = max(0.0, att);
	color        += att * att * (diffuse * NdotL + ambient);
	
	gl_FragColor = color;
}
